 /***********************************************************************
 *   Module : keyType.js     
 *   Description: This file contains the functionality necessary to 
 *                create and manipulate a keyType object. Note that 
 *                the keyInList is not a method of keyType object.
 * 
 *   Traceability to design : Represents the KeyType class.
 *
 *   Modification History:
 *   Author               Date            Version             Reason
 *   D.Palmer           09-Jan-01         initial               -             
 ***********************************************************************/

   // The maximum number of keys associated with each type of key.              
   var hksMAX_KEYS = 5;
   
   // The value used to indicate that a key is not in the key array.
   var keyNotInList = -1;


   /*
    *  This function constructs a new keyType object. 
    *  Params: 
    *     typeId: type String,
    *     description:  type String,
    *     keys: array of key elements.       
    */
    function keyType(typeId, description, keys) {
       this.typeId      = typeId;
       this.description = description;
       this.keys        = keys;
       this.removekey   = removeKey;
       this.getTopKey   = getTopKey;
       this.setTopKey   = setTopKey;
       this.addKey      = addKey;
    }


   /*
    *  This function determines if theKey exists in theList. 
    *  If it does, its position in the list is returned,
    *  otherwise -1 is returned.
    */
    function keyInList(theList, theKey) {
        // Remove all spaces from the start and end of the keyValue, 
        // this ensures it is compared correctly against the key in theList.
        theKey.value = theKey.value.replace(/^(\s|\xa0|&nbsp;)*(.*?)(\s|\xa0|&nbsp;)*$/,"$2");
        for (var index = 0; index < hksMAX_KEYS; index++) {
            if (theList.keys[index] != null) {
                if (theKey.typeId == theList.keys[index].typeId &
                    theKey.value == theList.keys[index].value   &
                    theKey.type == theList.keys[index].type) {
                        // key exists
                        return index;
                }
            }  
        }      
        return keyNotInList;
    }


    /*
     * This function removes theKey from the key array and arranges 
     * the remaining keys according to when they were last accessed,
     * with the most recent first.  
     */
     function removeKey(theKey) {

         /*
          * Holds the position of theKey in the list, if it exists.
          * Initialised such that no side effects occur should the 
          * key not exist.
          */
           var keyPosition = hksMAX_KEYS;
                  
           // Locate the position of key in list.
           keyPosition = keyInList(this, theKey);

           if (keyPosition != notInList) {
               // Shuffle keys up and remove last element. 
               for (i = keyPosition; i < (hksMAX_KEYS - 1); i++) {
                   this.keys[i] = this.keys[i+1];        
               } 
               this.keys[hksMAX_KEYS - 1] = null;                                                                
           }
       }


     /*
      * This function returns the last key used, that is the key residing at
      * the top of the list. 
      */
      function getTopKey() {
       
          if (this.keys[0] != null ) {
              return this.keys[0]; 
          }
          return null;  
      }

    
     /*
      *  This function sets the top key to newKey. If the key was not
      *  previously in the list all keys are shuffled down one place, 
      *  otherwise only the keys that were above it are shuffled down.
      */
      function setTopKey(newKey) {
          //alert('Set top key');
          // Holds a reference to the top key.
          var topKey = null;
         
          // Locate position of key in list.
          var keyPosition = keyInList(this, newKey);
            
                  
          if (keyPosition != keyNotInList) {
              topKey = this.keys[keyPosition];
              topKey.description = newKey.description;   // Update the description in case it has changed.
           }
           else {
               // key to be added to list and all keys need to be shuffled. 
               keyPosition = hksMAX_KEYS - 1;
               topKey = newKey;
           }
                     
           // Shuffle keys down one and set top key.
           for (var i = keyPosition; i > 0; i--) {                                               
               if (this.keys[i-1] != null) {
                   this.keys[i] = this.keys[i-1];
               }
           }      
           this.keys[0] = topKey;           
       }    
       
     /*  
      *  This function adds the key to the list. If the key was not
      *  previously in the list the key is added at the end of the list. 
      */
      function addKey(newKey) {
          // Locate position of key in list.
         var keyPosition = keyInList(this, newKey);
          
          //If key not already in the list add it to the list, otherwise do nothing
         if (keyPosition == keyNotInList) {
            // Checks the key list holds less than the max no. of keys
            if (this.keys.length < hksMAX_KEYS) {
                  var tempKeys = new Array(this.keys.length);
            
                  for(i=0; i < this.keys.length; i++) {
                     
                     tempKeys[i+1] = this.keys[i];
                  }  
                  tempKeys[0] = newKey;
                  this.keys = tempKeys;
                  
            }
            else {
               // Adds keys at end of list, shuffling the others up and losing the first.
              var tempKeys = new Array(this.keys.length);

              for(i=0; i < this.keys.length; i++) {
                if(i+1 < hksMAX_KEYS) {
                  tempKeys[i+1] = this.keys[i];
                }
              }   
              tempKeys[0] = newKey;
              this.keys = tempKeys;
            }
         }
         else {
            //Key already in list
            var keyPosition = keyInList(this, newKey);
            //alert('keyPosition='+keyPosition);
            
            var topKey = this.keys[keyPosition];
            topKey.description = newKey.description;
            
            // Shuffle keys down one and set top key.
            for (var i = keyPosition; i > 0; i--) {                                               
               if (this.keys[i-1] != null) {
                  this.keys[i] = this.keys[i-1];
                  }
            }      
            this.keys[0] = topKey;  
         }
      }                    

